/*****************************************************************************
MODULE:    LPC932 IAP Library Demonstration Project
CONTAINS:  Demonstration of how to use the LPC932 IAP Library
COPYRIGHT: Embedded Systems Academy, Inc. 2003 - www.esacademy.com
VERSION:   1.00, Andrew Ayre, 24-FEB-2003
-------------------------------------------------------------------------------
NOTES:   This code was written for use with an LPC932 using the on-chip
         RC oscillator (UCFG1 = 0x23) Adapt osc.c accordingly for other
		 oscillator/clock configurations.
         To see the output, connect UART to PC COM Port and connect using
         a terminal program such as Hyperterm. Settings:
	     8-N-1, 9600 baud, no flow control
	     Sections of code to program UCFG1 and the Boot Vector have been
	     included but commented, out to serve as examples.
CAUTION: For the IAP functionality of the device to work properly, brown-out
	     detection must be enabled in UCFG1, which may be set using a
	     parallel programmer or ISP programmer
	     (such as Flash Magic - www.esacademy.com/software/flashmagic)
	     It is NOT recommended to enable brown-out detection in
	     UCFG1 by using the IAP library to program UCFG1 as the programming of
		 UCFG1 may not be successful.
-------------------------------------------------------------------------------
HISTORY: 1.00, Andrew Ayre, 24-FEB-2003, First version
******************************************************************************/

// SFR description needs to be included
#include <reg932.h>
#include <stdio.h>
#include "osc.h"
#include "uart.h"
#include "lpc932iaplib.h"

// buffer used to write to Flash memory
unsigned char buffer[8];

/***********************************************************************
DESC:    Prints an error message based on the contents of iap_status
RETURNS: nothing
************************************************************************/
void print_error
  (
  void
  )
{
  if (iap_status & IAP_INTERRUPTED) printf("Interrupted\n");
  if (iap_status & IAP_SECURITY)    printf("Security violation\n");
  if (iap_status & IAP_ERROR)       printf("Error\n");
  if (iap_status & IAP_ABORT)       printf("Aborted\n");
} // print_error

/***********************************************************************
DESC:    main function
RETURNS: nothing
************************************************************************/
void main
  (
  void
  )
{
  unsigned char byte;
  unsigned long crc;

  // trim device to 7.303851MHz (0.9% error from 7.373MHz)
  //TRIM = 0x32;
  TRIM = 0x37;
  TRIM |= 0x40;

  // init oscillator
  osc_init();

  // init uart - 9600 baud
  uart_init();

  // init iap library
  iap_init(IAP_INTDISABLE);
  
  TI = 1;

  printf("LPC932 IAP Library Example\n");
  
  byte = iap_read_version();
  if (iap_status == IAP_OK)
    printf("IAP Version: 0x%2.2bX\n", byte);
  else
    print_error();

  /*
  iap_write(IAP_UCFG1, 0x23);
  if (iap_status == IAP_OK)
    printf("0x23 written to UCFG1.\n");
  else
    print_error();
  */

  byte = iap_read(IAP_UCFG1);
  if (iap_status == IAP_OK)
    printf("UCFG1: 0x%2.2bX\n", byte);
  else
    print_error();

  /*
  iap_write(IAP_BOOTVECTOR, 0x1E);
  if (iap_status == IAP_OK)
    printf("0x1E written to Boot Vector\n");
  else
    print_error();
  */

  byte = iap_read(IAP_BOOTVECTOR);
  if (iap_status == IAP_OK)
    printf("Boot Vector: 0x%2.2bX\n", byte);
  else
    print_error();

  iap_write(IAP_STATUSBYTE, 0x00);
  if (iap_status == IAP_OK)
    printf("0x00 written to Status Byte. ");
  else
    print_error();

  byte = iap_read(IAP_STATUSBYTE);
  if (iap_status == IAP_OK)
    printf("Status Byte: 0x%2.2bX.\n", byte);
  else
    print_error();

  byte = iap_read_code(0x1800);
  if (iap_status == IAP_OK)
    printf("Value at 0x1800: 0x%2.2bX\n", byte);
  else
    print_error();

  byte = iap_read_code(0x1840);
  if (iap_status == IAP_OK)
    printf("Value at 0x1840: 0x%2.2bX\n", byte);
  else
    print_error();

  buffer[0] = 0x11;
  iap_write_code(buffer, 0x1800, 1);
  if (iap_status == IAP_OK)
    printf("0x1800 programmed with 0x11\n");
  else
    print_error();

  byte = iap_read_code(0x1800);
  if (iap_status == IAP_OK)
    printf("Value at 0x1800: 0x%2.2bX\n", byte);
  else
    print_error();

  buffer[0] = 0x22;
  iap_write_code(buffer, 0x1840, 1);
  if (iap_status == IAP_OK)
    printf("0x1840 programmed with 0x22\n");
  else
    print_error();

  byte = iap_read_code(0x1840);
  if (iap_status == IAP_OK)
    printf("Value at 0x1840: 0x%2.2bX\n", byte);
  else
    print_error();

  crc = iap_read_sector_crc(0x1800);
  if (iap_status == IAP_OK)
    printf("Sector 6 CRC = %8.8lX\n", crc);
  else
    print_error();

  iap_erase(IAP_PAGE, 0x1840);
  if (iap_status == IAP_OK)
    printf("Page starting at 0x1840 erased\n");
  else
    print_error();

  byte = iap_read_code(0x1840);
  if (iap_status == IAP_OK)
    printf("Value at 0x1840: 0x%2.2bX\n", byte);
  else
    print_error();

  crc = iap_read_sector_crc(0x1800);
  if (iap_status == IAP_OK)
    printf("Sector 6 CRC = %8.8lX\n", crc);
  else
    print_error();
    
  iap_write(IAP_SECURITYBYTE6, 0x01);
  if (iap_status == IAP_OK)
    printf("0x01 written to Security Byte 6. ");	
  else
    print_error();

  byte = iap_read(IAP_SECURITYBYTE6);
  if (iap_status == IAP_OK)
    printf("Security Byte 6: 0x%2.2bX\n", byte);
  else
    print_error();

  iap_erase(IAP_SECTOR, 0x1800);
  if (iap_status == IAP_OK)
    printf("Sector 6 erased\n");
  else
    print_error();

  byte = iap_read(IAP_SECURITYBYTE6);
  if (iap_status == IAP_OK)
    printf("Security Byte 6: 0x%2.2bX\n", byte);
  else
    print_error();	

  byte = iap_read_code(0x1800);
  if (iap_status == IAP_OK)
    printf("Value at 0x1800: 0x%2.2bX\n", byte);
  else
    print_error();

  crc = iap_read_sector_crc(0x1800);
  if (iap_status == IAP_OK)
    printf("Sector 6 CRC = %8.8lX\n", crc);
  else
    print_error();

  crc = iap_read_global_crc();
  if (iap_status == IAP_OK)
    printf("Global CRC = %8.8lX\n", crc);
  else
    print_error();

  while(1);
} // main

